@@ -175,51 +175,82 @@ mp_set_from_double(double dx, MPNumber *z)
 
 
 void
-mp_set_from_integer(int x, MPNumber *z)
+mp_set_from_integer(int64_t x, MPNumber *z)
 {
+    int i;
+
     memset(z, 0, sizeof(MPNumber));
 
+    if (x == 0) {
+        z->sign = 0;
+        return;
+    }
+
     if (x < 0) {
         x = -x;
         z->sign = -1;
     }
     else if (x > 0)
         z->sign = 1;
-    else
-        z->sign = 0; /* Optimisation for indicating zero */
 
-    z->exponent = 1;
-    z->fraction[0] = x;
-    while (z->fraction[0] >= MP_BASE) {
-        int i;
-        for (i = z->exponent; i >= 0; i--)
-            z->fraction[i] = z->fraction[i-1];
-        z->fraction[0] = z->fraction[1] / MP_BASE;
-        z->fraction[1] = z->fraction[1] % MP_BASE;
+    while (x != 0) {
+        z->fraction[z->exponent] = x % MP_BASE;
+        x = x / MP_BASE;
         z->exponent++;
     }
+    for (i = 0; i < z->exponent / 2; i++) {
+        int t = z->fraction[i];
+        z->fraction[i] = z->fraction[z->exponent - i - 1];
+        z->fraction[z->exponent - i - 1] = t;
+    }
 }
 
 
 void
-mp_set_from_fraction(int i, int j, MPNumber *z)
+mp_set_from_unsigned_integer(uint64_t x, MPNumber *z)
 {
-    mp_gcd(&i, &j);
+    int i;
 
-    if (j == 0) {
+    memset(z, 0, sizeof(MPNumber));
+
+    if (x == 0) {
+        z->sign = 0;
+        return;
+    }
+    z->sign = 1;
+
+    while (x != 0) {
+        z->fraction[z->exponent] = x % MP_BASE;
+        x = x / MP_BASE;
+        z->exponent++;
+    }
+    for (i = 0; i < z->exponent / 2; i++) {
+        int t = z->fraction[i];
+        z->fraction[i] = z->fraction[z->exponent - i - 1];
+        z->fraction[z->exponent - i - 1] = t;
+    }
+}
+
+
+void
+mp_set_from_fraction(int64_t numerator, int64_t denominator, MPNumber *z)
+{
+    mp_gcd(&numerator, &denominator);
+
+    if (denominator == 0) {
         mperr("*** J == 0 IN CALL TO MP_SET_FROM_FRACTION ***\n");
         mp_set_from_integer(0, z);
         return;
     }
 
-    if (j < 0) {
-        i = -i;
-        j = -j;
+    if (denominator < 0) {
+        numerator = -numerator;
+        denominator = -denominator;
     }
 
-    mp_set_from_integer(i, z);
-    if (j != 1)
-        mp_divide_integer(z, j, z);
+    mp_set_from_integer(numerator, z);
+    if (denominator != 1)
+        mp_divide_integer(z, denominator, z);
 }
 
 
@@ -254,18 +285,17 @@ mp_set_from_random(MPNumber *z)
 }
 
 
-int
+int64_t
 mp_cast_to_int(const MPNumber *x)
 {
-    int i, j, x2, xs, ret_val = 0;
+    int i, j;
+    int64_t ret_val = 0;
 
     /* RETURN 0 IF X = 0 OR IF NUMBER FRACTION */
-    xs = x->sign;
-    if (xs == 0 || x->exponent <= 0)
+    if (x->sign == 0 || x->exponent <= 0)
         return 0;
 
-    x2 = x->exponent;
-    for (i = 0; i < x2; i++) {
+    for (i = 0; i < x->exponent; i++) {
         int izs;
         izs = ret_val;
         ret_val = MP_BASE * ret_val;
@@ -277,11 +307,9 @@ mp_cast_to_int(const MPNumber *x)
             return 0;
     }
 
-    /*  CHECK THAT RESULT IS CORRECT (AN UNDETECTED OVERFLOW MAY
-     *  HAVE OCCURRED).
-     */
+    /*  CHECK THAT RESULT IS CORRECT (AN UNDETECTED OVERFLOW MAY HAVE OCCURRED). */
     j = ret_val;
-    for (i = x2 - 1; i >= 0; i--) {
+    for (i = x->exponent - 1; i >= 0; i--) {
         int j1, kx;
 
         j1 = j / MP_BASE;
@@ -296,7 +324,7 @@ mp_cast_to_int(const MPNumber *x)
         return 0;
 
     /* RESULT CORRECT SO RESTORE SIGN AND RETURN */
-    return xs * ret_val;
+    return x->sign * ret_val;
 
     /* Old comment about returning zero: */
     /*  HERE OVERFLOW OCCURRED (OR X WAS UNNORMALIZED), SO
@@ -305,6 +333,48 @@ mp_cast_to_int(const MPNumber *x)
 }
 
 
+uint64_t
+mp_cast_to_unsigned_int(const MPNumber *x)
+{
+    int i, j;
+    uint64_t ret_val = 0;
+
+    /* RETURN 0 IF X <= 0 OR IF NUMBER FRACTION */
+    if (x->sign <= 0 || x->exponent <= 0)
+        return 0;
+
+    for (i = 0; i < x->exponent; i++) {
+        int izs;
+        izs = ret_val;
+        ret_val = MP_BASE * ret_val;
+        if (i < MP_T)
+            ret_val += x->fraction[i];
+
+        /* CHECK FOR SIGNS OF INTEGER OVERFLOW */
+        if (ret_val <= 0 || ret_val <= izs)
+            return 0;
+    }
+
+    /*  CHECK THAT RESULT IS CORRECT (AN UNDETECTED OVERFLOW MAY HAVE OCCURRED). */
+    j = ret_val;
+    for (i = x->exponent - 1; i >= 0; i--) {
+        int j1, kx;
+
+        j1 = j / MP_BASE;
+        kx = 0;
+        if (i < MP_T)
+            kx = x->fraction[i];
+        if (kx != j - MP_BASE * j1)
+            return 0;
+        j = j1;
+    }
+    if (j != 0)
+        return 0;
+
+    return ret_val;
+}
+
+
 static double
 mppow_ri(float ap, int bp)
 {
@@ -441,7 +511,7 @@ mp_cast_to_double(const MPNumber *x)
 
 
 static void
-mp_cast_to_string_real(const MPNumber *x, int base, int accuracy, int trim_zeroes, int force_sign, GString *string)
+mp_cast_to_string_real(const MPNumber *x, int base, int accuracy, bool trim_zeroes, bool force_sign, GString *string)
 {
     static char digits[] = "0123456789ABCDEF";
     MPNumber number, integer_component, fractional_component, temp;
@@ -531,7 +601,7 @@ mp_cast_to_string_real(const MPNumber *x, int base, int accuracy, int trim_zeroe
 
 
 void
-mp_cast_to_string(const MPNumber *x, int base, int accuracy, int trim_zeroes, char *buffer, int buffer_length)
+mp_cast_to_string(const MPNumber *x, int base, int accuracy, bool trim_zeroes, char *buffer, int buffer_length)
 {
     MPNumber x_real, x_im;
     GString *string;
@@ -655,7 +725,7 @@ ends_with(const char *start, const char *end, const char *word)
 }
 
 
-int
+bool
 mp_set_from_string(const char *str, MPNumber *z)
 {
     int i, base, negate = 0, multiplier = 0, base_multiplier = 1;
@@ -703,7 +773,7 @@ mp_set_from_string(const char *str, MPNumber *z)
     mp_set_from_integer(0, z);
     while ((i = char_val((char **)&c, base)) >= 0) {
         if (i > base)
-            return 1;
+            return true;
         mp_multiply_integer(z, base, z);
         mp_add_integer(z, i, z);
     }
@@ -742,7 +812,7 @@ mp_set_from_string(const char *str, MPNumber *z)
     }
 
     if (c != end) {
-        return 1;
+        return true;
     }
 
     if (multiplier != 0) {
@@ -755,5 +825,5 @@ mp_set_from_string(const char *str, MPNumber *z)
     if (negate == 1)
         mp_invert_sign(z, z);
 
-    return 0;
+    return false;
 }
